@@ -2,7 +2,7 @@
 #
 # (c) 2005, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: BerkeleyDB.pm,v 0.81 2009/03/05 00:09:34 acorliss Exp $
+# $Id: BerkeleyDB.pm,v 0.83 2010/04/15 23:15:38 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -27,7 +27,7 @@ use Paranoid::Filesystem qw(pmkdir);
 use BerkeleyDB;
 use Carp;
 
-($VERSION) = ( q$Revision: 0.81 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 0.83 $ =~ /(\d+(?:\.(\d+))+)/sm );
 
 #####################################################################
 #
@@ -36,14 +36,21 @@ use Carp;
 #####################################################################
 
 sub new (@) {
-    my $class = shift;
-    my %args  = @_;
-    my %init  = (
+
+    # Purpose:  Instantiates a new object of this class
+    # Returns:  Object reference if successful, undef otherwise
+    # Usage:    $obj = Paranoid::BerkeleyDB->new(
+    #                   DbDir   = $dir,
+    #                   DbName  = $name,
+    #                   );
+
+    my ( $class, %args ) = @_;
+    my %init = (
         DbDir  => undef,
         DbName => undef,
         Dbs    => {},
         DbEnv  => undef,
-    );
+        );
     my $dbdir = defined $args{DbDir}  ? $args{DbDir}  : 'undef';
     my $dbnm  = defined $args{DbName} ? $args{DbName} : 'undef';
     my ( $self, $tmp );
@@ -55,13 +62,13 @@ sub new (@) {
     if ( defined $dbdir and defined $dbnm ) {
 
         # Create the directory (and let umask determine the permissions)
-        if ( pmkdir( $dbdir, 0777 ) ) {
+        if ( pmkdir($dbdir) ) {
 
             # Create lock file and lock it while doing initialization.  I
             # know, this isn't ideal when creating temporary objects that
             # need only read access, but it's the only way to avoid race
             # conditions if this is the process that creates the database.
-            plock( "$dbdir/db.lock", undef, 0666 );
+            plock( "$dbdir/db.lock", 'write', 0666 );
 
             # Create and bless the object reference
             @init{qw(DbDir DbName)} = ( $dbdir, $dbnm );
@@ -76,8 +83,9 @@ sub new (@) {
                     $tmp = BerkeleyDB::Env->new(
                         '-Home'    => $dbdir,
                         '-ErrFile' => \*STDERR,
-                        '-Flags'   => DB_CREATE | DB_INIT_CDB | DB_INIT_MPOOL,
-                    ) )
+                        '-Flags' => DB_CREATE | DB_INIT_CDB | DB_INIT_MPOOL |
+                            DB_CDB_ALLDB,
+                        ) )
                 ) {
 
                 # Success! Now, create the database
@@ -86,7 +94,7 @@ sub new (@) {
                     '-Filename' => $dbnm,
                     '-Env'      => $self->{DbEnv},
                     '-Flags'    => DB_CREATE,
-                );
+                    );
 
                 # Check if creating the db was successful
                 if ( defined $tmp ) {
@@ -132,6 +140,11 @@ sub new (@) {
 }
 
 sub addDb ($$) {
+
+    # Purpose:  Adds a new named database to the current environment
+    # Returns:  True/false
+    # Usage:    $rv = $db->addDb( 'foo.db' );
+
     my $self  = shift;
     my $dbnm  = shift;
     my $dbdir = $self->{DbDir};
@@ -152,7 +165,10 @@ sub addDb ($$) {
             '-Filename' => $dbnm,
             '-Env'      => $self->{DbEnv},
             '-Flags'    => DB_CREATE,
-        );
+            );
+
+        # Release lock
+        punlock("$dbdir/db.lock");
 
         # Store & report the result
         $rv = defined $db ? 1 : 0;
@@ -163,9 +179,6 @@ sub addDb ($$) {
             Paranoid::ERROR =
                 pdebug( "failed to add new database: $dbnm", PDLEVEL1 );
         }
-
-        # Release lock
-        punlock("$dbdir/db.lock");
     }
 
     pOut();
@@ -175,6 +188,12 @@ sub addDb ($$) {
 }
 
 sub getVal ($$;$) {
+
+    # Purpose:  Returns the associated value for the requested key.
+    # Returns:  String if key exists, undef otherwise
+    # Usage:    $db->getVal( $key );
+    # Usage:    $db->getVal( $key, $dbName );
+
     my $self  = shift;
     my $key   = shift;
     my $db    = shift;
@@ -199,14 +218,14 @@ sub getVal ($$;$) {
         # Requested database exists
         #
         # Lock database for read mode
-        plock( "$dbdir/db.lock", 'shared' );
+        #plock( "$dbdir/db.lock", 'shared' );
 
         unless ( $$dref{$db}->db_get( $key, $val ) == 0 ) {
             pdebug( "no such key exists ($key)", PDLEVEL2 );
         }
 
         # Unlock database
-        punlock("$dbdir/db.lock");
+        #punlock("$dbdir/db.lock");
 
     } else {
 
@@ -224,6 +243,13 @@ sub getVal ($$;$) {
 }
 
 sub setVal ($$;$$) {
+
+    # Purpose:  Associates the key with the passed value or, if the
+    #           value is undefined, deletes any existing key.
+    # Returns:  True/false
+    # Usage:    $db->setVal( $key, $value );
+    # Usage:    $db->setVal( $key, $value, $dbName );
+
     my $self  = shift;
     my $key   = shift;
     my $val   = shift;
@@ -251,11 +277,10 @@ sub setVal ($$;$$) {
         # Requested database exists
         #
         # Make sure key is defined
-        if ( defined $key ) {
+        if ( defined $key and defined( $lock = $$dref{$db}->cds_lock ) ) {
 
             # Lock database for write mode
-            plock("$dbdir/db.lock");
-            $lock = $$dref{$db}->cds_lock;
+            #plock("$dbdir/db.lock");
 
             # Check whether setting a new record or deleting one
             if ( defined $val ) {
@@ -274,7 +299,8 @@ sub setVal ($$;$$) {
             # Unlock database
             $$dref{$db}->db_sync;
             $lock->cds_unlock;
-            punlock("$dbdir/db.lock");
+
+            #punlock("$dbdir/db.lock");
 
         } else {
 
@@ -297,15 +323,25 @@ sub setVal ($$;$$) {
     return $rv;
 }
 
-sub getKeys ($;$) {
-    my $self  = shift;
-    my $db    = shift;
-    my $d     = defined $db ? $db : 'undef';
-    my $dref  = $self->{Dbs};
-    my $dbdir = $self->{DbDir};
+sub getKeys ($;$$) {
+
+    # Purpose:  Returns a list of all the keys in the database and
+    #           optionally runs a subroutine over each record.
+    # Returns:  List of keys.
+    # Usage:    @keys = $db->getKeys;
+    # Usage:    @keys = $db->getKeys( $dbName );
+    # Usage:    @keys = $db->getKeys( $dbName, undef );
+
+    my $self   = shift;
+    my $db     = shift;
+    my $subRef = shift;
+    my $d      = defined $db ? $db : 'undef';
+    my $s      = defined $subRef ? $subRef : 'undef';
+    my $dref   = $self->{Dbs};
+    my $dbdir  = $self->{DbDir};
     my ( $cursor, $key, $val, @keys );
 
-    pdebug( "entering w/($d)", PDLEVEL1 );
+    pdebug( "entering w/($d)($s)", PDLEVEL1 );
     pIn();
 
     # Set the default database name if it wasn't passed
@@ -318,21 +354,39 @@ sub getKeys ($;$) {
     if ( exists $$dref{$db} ) {
 
         # Lock database for read mode
-        plock( "$dbdir/db.lock", 'shared' );
+        #plock( "$dbdir/db.lock", 'shared' );
 
         # Retrieve all the keys
         $key = $val = '';
-        $cursor = $$dref{$db}->db_cursor;
+        $cursor =
+            defined $subRef
+            ? $$dref{$db}->db_cursor(DB_WRITECURSOR)
+            : $$dref{$db}->db_cursor;
         while ( $cursor->c_get( $key, $val, DB_NEXT ) == 0 ) {
-            push @keys, $key if defined $key;
+
+            if ( defined $key ) {
+
+                #plock( "$dbdir/db.lock", 'write' );
+
+                # The method was passed a subroutine reference, so
+                # unlock the database and call the routine
+                &$subRef( $self, $key, $val ) if defined $subRef;
+
+                # Save the key;
+                push @keys, $key;
+            }
+
+            #plock( "$dbdir/db.lock", 'shared' );
         }
         $cursor->c_close;
+        undef $cursor;
 
         # Unlock database
-        punlock("$dbdir/db.lock");
+        #punlock("$dbdir/db.lock");
+
+    } else {
 
         # Report invalid database
-    } else {
         Paranoid::ERROR =
             pdebug( "attempted to access a nonexistent database ($db)",
             PDLEVEL1 );
@@ -345,6 +399,12 @@ sub getKeys ($;$) {
 }
 
 sub purgeDb ($;$) {
+
+    # Purpose:  Empties the database.
+    # Returns:  True/false
+    # Usage:    $rv = $db->purgeDb;
+    # Usage:    $rv = $db->purgeDb( $dbName );
+
     my $self  = shift;
     my $db    = shift;
     my $d     = defined $db ? $db : 'undef';
@@ -392,6 +452,11 @@ sub purgeDb ($;$) {
 }
 
 sub listDbs ($) {
+
+    # Purpose:  Returns a list of all named database
+    # Returns:  List of database names
+    # Usage:    @dbs = $db->listDbs;
+
     my $self = shift;
     my $dref = $self->{Dbs};
     my @dbs  = keys %$dref;
@@ -415,7 +480,6 @@ sub DESTROY {
     foreach ( keys %$dref ) {
         if ( defined $$dref{$_} ) {
             pdebug( "sync/close $_", PDLEVEL2 );
-            $$dref{$_}->db_sync;
             $$dref{$_}->db_close;
             delete $$dref{$_};
         }
@@ -441,7 +505,7 @@ Paranoid::BerkeleyDB -- BerkeleyDB concurrent-access Object
 
 =head1 VERSION
 
-$Id: BerkeleyDB.pm,v 0.81 2009/03/05 00:09:34 acorliss Exp $
+$Id: BerkeleyDB.pm,v 0.83 2010/04/15 23:15:38 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -458,6 +522,8 @@ $Id: BerkeleyDB.pm,v 0.81 2009/03/05 00:09:34 acorliss Exp $
 
   @keys = $db->getKeys();
   @keys = $db->getKeys($dbname);
+  @keys = $db->getKeys(undef, \&sub);
+  @keys = $db->getKeys($dbname, \&sub);
 
   $db->purgeDb();
   $db->purgeDb($dbname);
@@ -547,8 +613,18 @@ be set in B<Paranoid::ERROR>.
 
   @keys = $db->getKeys();
   @keys = $db->getKeys($dbname);
+  @keys = $db->getKeys(undef, \&sub);
+  @keys = $db->getKeys($dbname, \&sub);
 
-This method returns all of the keys in the requested database, in hash order.
+If this method is called without the optional subroutine reference it will
+return all the keys in the hash in hash order.  If a subroutine reference is
+called it will be called as each key/value pair is iterated over with three
+arguments:
+
+    &$subRef($self, $key, $value);
+
+with $self being a handle to the current Paranoid::BerkeleyDB object.  You may
+use this object handle to perform other database operations as needed.
 
 =head2 purgeDb
 
@@ -566,6 +642,11 @@ requested.
 
 This method returns a list of databases accessible by this object.
 
+=head2 DESTROY
+
+A DESTROY method is provided which should sync and close an open database, as
+well as release any locks.
+
 =head1 DEPENDENCIES
 
 =over
